import inspect
import os
import sys

__version__ = '0.0.10'

real_path = os.path.dirname(os.path.abspath(__file__)).replace("\\","/")
sys.path.append(real_path)

try:
    from service.calculate.profit import Profit
    from service.date.date import Date
    from service.db.db import DB
    from service.format.format import Format
except ImportError as e:
    print(e," 추가할 수 없습니다.")
    exit(1)


__all__ = [name for name, obj in locals().items()
           if not (name.startswith('_') or inspect.ismodule(obj))]


from dataclasses import dataclass

import pymysql
import sqlalchemy as sqlalchemy
from sqlalchemy.engine.url import URL
import pandas as pd


@dataclass
class DBProperty:
    database_name: str = None
    user: str = None
    password: str = None
    host: str = None


class DB:
    database_name = None
    sqlalchemy_connector = None
    db_property = None

    def __init__(self, db_property: DBProperty):
        self.db_property = db_property
        self.sqlalchemy_connector = self._get_sqlalchemy_connector()
        self.pymysql_connector = self._get_pymysql_connector()
        self.pymysql_cursor = self.pymysql_connector.cursor()

    def _get_sqlalchemy_connector(self):
        pymysql.install_as_MySQLdb()
        conn = sqlalchemy.create_engine(URL(
            drivername='mysql',
            username=self.db_property.user,
            password=self.db_property.password,
            host=self.db_property.host,
            database=self.db_property.database_name
        ))
        return conn

    def _get_pymysql_connector(self):
        return pymysql.connect(
            host=self.db_property.host,
            user=self.db_property.user,
            password=self.db_property.password,
            db=self.db_property.database_name,
            charset='utf8'
        )

    def con_close(self):
        self.pymysql_connector.close()
        self.sqlalchemy_connector.dispose()

    def get_sqlalchemy_connector(self):
        return self.sqlalchemy_connector

    def get_pymysql_connector(self):
        return self.pymysql_connector

    def get_tables(self) -> list:
        sql = "show tables"
        table_df = pd.read_sql(sql=sql, con=self.sqlalchemy_connector)
        return table_df[table_df.columns[0]].to_list()

    def drop_table(self, table_name):
        sql = f"drop table {table_name}"
        self.pymysql_cursor.execute(sql)

    def drop_all_table(self):
        list(map(self.drop_table, self.get_tables()))



import datetime
from datetime import timedelta
from dateutil.relativedelta import relativedelta

class Date:
    @staticmethod
    def generate_month(start: str, end: str, interval: int):
        month_list = []
        start_date = datetime.datetime.strptime(start, "%Y-%m-%d")
        end_date = datetime.datetime.strptime(end, "%Y-%m-%d")

        tmp_date = start_date
        while tmp_date != end_date:
            month_list.append(tmp_date.strftime("%Y-%m-%d"))
            tmp_date = tmp_date + relativedelta(months=interval)
        month_list.append(end_date.strftime("%Y-%m-%d"))
        return month_list

    @staticmethod
    def generate_day(start: str, end: str, interval: int):
        day_list = []
        start_date = datetime.datetime.strptime(start, "%Y-%m-%d")
        end_date = datetime.datetime.strptime(end, "%Y-%m-%d")

        tmp_date = start_date
        while tmp_date != end_date:
            day_list.append(tmp_date.strftime("%Y-%m-%d"))
            tmp_date = tmp_date + timedelta(days=interval)
        day_list.append(end_date.strftime("%Y-%m-%d"))
        return day_list




from setuptools import setup, find_packages


setup(
    name='quantservice',
    version='0.0.12',
    url='https://github.com/dudals3844/quant_service',
    author='ChoiYoungMin',
    author_email='dudals3844@gmail.com',
    description='Quant Service',
    packages=find_packages(exclude=['test']),
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=['pandas', 'pymysql', 'sqlalchemy', 'cryptography', 'python-dateutil'],
    zip_safe=False,
    classifiers=[
        'License :: OSI Approved :: MIT License'
    ]
)


from pandas import Series
import numpy as np

class Profit:
    @staticmethod
    def get(series: Series) -> np.float64:
        series += 1
        return series.prod() - 1

    @staticmethod
    def get_cum_prod(series: Series) -> Series:
        series += 1
        return series.cumprod() - 1

